import router from './router'
import store from './store'
import { Message } from 'element-ui'
import NProgress from 'nprogress' // progress bar
import 'nprogress/nprogress.css' // progress bar style
import { getToken, getUserId } from '@/utils/auth' // get token from cookie
import getPageTitle from '@/utils/get-page-title'

NProgress.configure({ showSpinner: false }) // NProgress Configuration

const whiteList = ['/login', '/auth-redirect', '/file-manage', '/my-file-manage'] // no redirect whitelist

// store.commit('permission/SET_ROUTES', [])

router.beforeEach(async(to, from, next) => {
  // start progress bar
  NProgress.start()

  // set page title
  document.title = getPageTitle(to.meta.title)
  if (to.path === '/login') {
    next()
     // if is logged in, redirect to the home page
    // next({ path: '/login' })
    // NProgress.done() // hack: https://github.com/PanJiaChen/vue-element-admin/pull/2939
    return
  }
  const hasToken = getToken() && getUserId()
  //
  if( hasToken ){
     next()
  } else {
    if (whiteList.indexOf(to.path) !== -1) {
      // in the free login whitelist, go directly
      next()
    } else {
      // other pages that do not have permission to access are redirected to the login page.
      next(`/login?redirect=${to.path}`)
      NProgress.done()
    }
  }
})

router.afterEach(() => {
  // finish progress bar
  NProgress.done()
})
