import Vue from 'vue'
import Router from 'vue-router'

import main from '@/pages/other/index'

Vue.use(Router)

/* Layout */
import Layout from '@/layout'

// /* Router Modules */
// import componentsRouter from './modules/components'
// import chartsRouter from './modules/charts'
// import tableRouter from './modules/table'
// import nestedRouter from './modules/nested'

/**
 * Note: sub-menu only appear when route children.length >= 1
 * Detail see: https://panjiachen.github.io/vue-element-admin-site/guide/essentials/router-and-nav.html
 *
 * hidden: true                   if set true, item will not show in the sidebar(default is false)
 * alwaysShow: true               if set true, will always show the root menu
 *                                if not set alwaysShow, when item has more than one children route,
 *                                it will becomes nested mode, otherwise not show the root menu
 * redirect: noRedirect           if set noRedirect will no redirect in the breadcrumb
 * name:'router-name'             the name is used by <keep-alive> (must set!!!)
 * meta : {
    roles: ['admin','editor']    control the page roles (you can set multiple roles)
    title: 'title'               the name show in sidebar and breadcrumb (recommend set)
    icon: 'svg-name'/'el-icon-x' the icon show in the sidebar
    noCache: true                if set true, the page will no be cached(default is false)
    affix: true                  if set true, the tag will affix in the tags-view
    breadcrumb: false            if set false, the item will hidden in breadcrumb(default is true)
    activeMenu: '/example/list'  if set path, the sidebar will highlight the path you set
  }
 */

/**
 * constantRoutes
 * a base page that does not have permission requirements
 * all roles can be accessed
 */
export const constantRoutes = [
    {
      path: '/login',
      component: () => import('@/pages/login/index'),
      hidden: true
    },
    {
      path: '/auth-redirect',
      component: () => import('@/pages/login/auth-redirect'),
      hidden: true
    },
    {
      path: '/200',
      component: () => import('@/pages/error-page/200'),
      name: 'page200',
      hidden: true
    },
    {
      path: '/404',
      component: () => import('@/pages/error-page/404'),
      hidden: true
    },
    {
      path: '/401',
      component: () => import('@/pages/error-page/401'),
      hidden: true
    },
    {
      path: '/file-manage',
      component: () => import('@/pages/other/file-manage'),
      hidden: true
    },
    {
      path: '/',
      redirect: '/file-manage',
      hidden: true
    },
    {
      path: '/user',
      name: 'user',
      component: main,
      children: [{
          path: '/apply-list',
          component: () => import('@/pages/other/apply-list'),
          hidden: true
        },
        {
          path: '/reply-list',
          component: () => import('@/pages/other/reply-list'),
          hidden: true
        },
        {
          path: '/file-search',
          component: () => import('@/pages/other/file-search'),
          hidden: true
        },
        {
          path: '/my-file',
          component: () => import('@/pages/other/my-file'),
          keepAlive:true,
          hidden: true
        },
        {
          path: '/reset',
          name:'reset',
          component: () => import('@/pages/other/reset'),
          hidden: true
        },
        {
          path: '/file-add',
          component: () => import('@/pages/other/file-add'),
          hidden: true
        },
        {
          path: '/file-confirm',
          component: () => import('@/pages/other/file-confirm'),
          hidden: true
        },
        {
          path: '/file-publish',
          component: () => import('@/pages/other/file-publish'),
          hidden: true
        },
        {
          path: '/file-info',
          component: () => import('@/pages/other/file-info'),
          hidden: true
        }
      ],
    }
  ]

    /**
     * asyncRoutes
     * the routes that need to be dynamically loaded based on user roles
     */
    export const asyncRoutes = [
      // 404 page must be placed at the end !!!
      {
        path: '*',
        redirect: '/404',
        hidden: true
      }
    ]

    const createRouter = () => new Router({
      // mode: 'history', // require service support
      scrollBehavior: () => ({
        y: 0
      }),
      routes: constantRoutes
    })

    const router = createRouter()

    // Detail see: https://github.com/vuejs/vue-router/issues/1234#issuecomment-357941465
    export function resetRouter() {
      const newRouter = createRouter()
      router.matcher = newRouter.matcher // reset router
    }

    export default router
